#! /usr/bin/env ruby

module NFSIni
  def self.read_io io
    sect = nil
    out = {}
    io.each do |l|
      l.chomp!
      if l =~ /\A[[:space:]]*'/ # comment
        next
      elsif l =~ /\A[[:space:]]*\[(.*)\][[:space:]]*\z/ # section
        sect = $1
      elsif l =~ /\A([^=]+)=(.*)\z/ # k=v
        (out[sect] ||= {})[$1] = $2
      elsif l =~ /\A[[:space:]]*\z/ # empty
        next
      else
        fail "Invalid ini line #{l.inspect}"
      end
    end
    out
  end

  def self.write_sect io, sect
    sect.each {|k,v| io.puts "#{k}=#{v}" }
  end

  def self.write_io io, ini
    if ini[nil]
      ini = ini.dup
      write_sect io, ini.delete(nil)
    end

    ini.each do |k,v|
      io.puts "[#{k}]"
      write_sect io, v
    end
    nil
  end

  def self.read_file(fn) = File.open(fn, 'r') {|f| read_io f }
  def self.write_file(fn, ini) = File.open(fn, 'w') {|f| write_io f, ini }
end

module NFS6Text
  ENC = Encoding::UTF_16LE
  ZERO = "\0".encode ENC
  def self.read buf
    buf.force_encoding ENC
    n = buf.unpack1('V') / 4
    ary = buf.byteslice(n*4..).split ZERO, -1
    fail unless ary.pop.empty?
    fail unless ary.size == n
    ary
  end

  def self.write ary, io
    ary = ary.map {|x| x.encode ENC }
    off = ary.size*4
    ofm = ary.map do |m|
      res = off
      off += m.bytesize + 2
      res
    end

    io.write ofm.pack 'V*'
    io.write ary.join ZERO
    io.write ZERO
  end

  def self.calc_car_map
    base_dir = File.join __dir__, 'ovrlay/base'
    cars = NFSIni.read_file File.join base_dir, 'cars/cars.ini'
    dbg = NFS6Text.read File.read 'ovrlay/base/text/unicodetext.dbg'
    map = dbg.each_with_index.to_h {|k,i| [k.encode(Encoding::UTF_8), i] }

    cars.filter_map do |_,sect|
      next nil if sect['traffic'] == 'true'
      name = sect['name'] or fail
      base_txt = sect['displayname'] or fail

      ids = [map[base_txt] || fail, map["#{base_txt}2Line"],
             map["#{base_txt}Short"]].compact
      nfs_txt = base_txt.sub /\AkTxt/, 'kTxtNFS'
      nfs_ids = [map[nfs_txt], map["#{nfs_txt}2Line"]].compact
      [name, [ids, nfs_ids]]
    end.to_h
  end
  CAR_MAP = calc_car_map.freeze
end

return unless __FILE__ == $0
# require 'ap'

# $stdin.binmode
# ary = NFS6Text.read $stdin.read
# ap ary.map {|x| x.encode Encoding::UTF_8 }

dbg = NFS6Text.read File.read 'ovrlay/base/text/unicodetext.dbg'
eng = NFS6Text.read File.read 'ovrlay/base/text/unicodetext.eng'

out = eng.zip(dbg).each_with_index.map do |(e,d),i|
  "#{i}(#{d.encode(Encoding::UTF_8).gsub /\AkTxt/, ''}) #{e.encode(Encoding::UTF_8).inspect}"
end
File.open 'mount/text/unicodetext.eng', 'w' do |f|
  NFS6Text.write out, f
end

# select: Murcielago2Line -- NFSMurcielago2Line
# start/end screen: Murcielago
# but also: BarchettaShort
